package irrigation;

import java.util.HashSet;
import java.util.List;

import device.SensorDevice;
import device.ValveDevice;
import static tools.HelperFunctions.*;

public class ScenarioHelper {

	public static HashSet<Zone> getZones(Irrigator i)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		return extractField(i, "zones");
	}

	public static HashSet<Valve> getValves(Irrigator i, String zoneName)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		for (Zone z : getZones(i)) {
			if (z.getID().equals(zoneName))
				return extractField(z, "valves");
		}

		throw new IllegalArgumentException("Zone " + zoneName
				+ " does not exist.");
	}
	
	public static List<String> getFailedDevices(Irrigator i)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		return extractField(i, "failedDevices");
	}


	public static Sensor getSensor(Irrigator i, String zoneName)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		for (Zone z : getZones(i)) {
			if (z.getID().equals(zoneName))
				return extractField(z, "sensor");
		}
		throw new IllegalArgumentException("Zone " + zoneName
				+ " does not exist.");
	}

	public static ValveDevice getValveDevice(Valve v)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		return extractField(v, "device");
	}

	public static SensorDevice getSensorDevice(Sensor s)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		return extractField(s, "device");
	}

}
