package irrigation;

import static irrigation.ScenarioHelper.getValves;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static tools.HelperFunctions.extractField;

import org.junit.Test;

import simulation.SimTime;
import util.Day;
import device.StoreFailureException;

/**
 * Tests the basic flow of AquaLush's first use case.
 */
public class UseCaseOneBasicFlowTest extends UseCaseOneTest {

	@Override
	public void stepOne() {
		// 1. Operator sets the mode from manual to automatic.

		// Leave the "Control Irrigation" screen ...
		buttons[7].doClick();
		// ... and change the irrigation mode.
		buttons[3].doClick();
	}

	public void prepareUseCase() {
		// Initialization - set AquaLush to manual mode and open
		// some of the valves.

		SimTime.instance().setDay(Day.MONDAY);
		SimTime.instance().setHour(12);
		SimTime.instance().setMinute(00);
		SimTime.instance().setSecond(00);

		// Change mode to manual, ...
		buttons[3].doClick();
		// ... access the "Control Irrigation" screen, ...
		buttons[7].doClick();
		// ... select valve "V05", ...
		for (int i = 0; i < 3; i++)
			buttons[3].doClick();
		// ... open a valve, ...
		buttons[2].doClick();
		// ... propagate this change to the other valves, ...
		buttons[4].doClick();

		// Let a few minutes pass
		for (int i = 0; i < 3; i++)
			clock.tick();
	}

	@Test
	public void testStepOne() {

		assertEquals(Mode.AUTOMATIC, irrigator.getMode());
	}

	@Test
	public void testStepTwo() {

		// 2. AquaLush turns off any Valves that may be on.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}

		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
	}

	@Test
	public void testStepThree() {

		// 3. AquaLush records the mode as automatic, ...

		try {
			assertEquals(Mode.AUTOMATIC.toString(), store.getData("mode", "manual"));
		} catch (StoreFailureException e) {
			fail();
		}

		// ... confirms the change ...
		try {
			assertTrue(extractField(uiController, "crntScrn") == extractField(
					uiController, "autoMainScrn"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// ... and waits for the next irrigation time.

		clock.tick();

		try {
			assertNull(extractField(irrigator, "cycle"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

	}

}
