package irrigation;

import static irrigation.ScenarioHelper.getValves;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.fail;
import simulation.SimTime;
import util.Day;

/**
 * Tests extension 1a of the first use case.
 */
public class UseCaseOneExtension1ATest extends UseCaseOneTest {

	public void prepareUseCase() {

		// An automatic irrigation cycle
		// starts.

		SimTime.instance().setDay(Day.MONDAY);
		SimTime.instance().setMinute(00);
		SimTime.instance().setSecond(00);
		SimTime.instance().setHour(2);

		// Let a few minutes pass
		for (int i = 0; i < 5; i++)
			clock.tick();

	}

	@Override
	public void stepOne() {
		// 1a Operator sets the mode from automatic to manual.

		buttons[3].doClick();

	}

	@Override
	public void testStepOne() {

		assertEquals(Mode.MANUAL, irrigator.getMode());

	}

	@Override
	public void testStepTwo() {

		// 1a1. AquaLush closes all Valves.

		try {
			for (Valve v : getValves(irrigator, "Z1")){
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")){
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")){
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}

		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

	}

	@Override
	public void testStepThree() {

		// 1a2. AquaLush records the mode as manual and confirms the change, and
		// the use case ends.

		try {
			assertEquals(Mode.MANUAL.toString(), store.getData("mode", "automatic"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

	}

}
