package irrigation;

import static irrigation.ScenarioHelper.getFailedDevices;
import static irrigation.ScenarioHelper.getValves;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static tools.HelperFunctions.extractField;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JCheckBox;

import org.junit.Test;

import device.StoreFailureException;

/**
 * Tests extension 2A of the first use case.
 */
public class UseCaseOneExtension2ABasicFlowTest extends UseCaseOneBasicFlowTest {

	@Override
	public void prepareUseCase(){

		super.prepareUseCase();

		// Suddenly, two valves fail.
		JCheckBox v06 = null;
		JCheckBox v07 = null;
		for (JCheckBox c : checkboxes)
			if (c.getText().equals("V06"))
				v06 = c;
			else if (c.getText().equals("V07"))
				v07 = c;

		v06.doClick();
		v07.doClick();
	}

	@Override
	public void testStepTwo() {
		
		// 2. AquaLush turns off any Valves that may be on.
		
		try {
			for (Valve v : getValves(irrigator, "Z1")){
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			
			// The failed valves should still be open (they failed while closing)
			
			for (Valve v : getValves(irrigator, "Z2"))
				// two valves could not be closed because they failed
				if (v.getID().equals("V07") || v.getID().equals("V06")){
					assertTrue(v.isOpen());
					assertTrue(simulation.getValve(v.getID()).isOpen());
				}
				else {
					assertFalse(v.isOpen());
					assertFalse(simulation.getValve(v.getID()).isOpen());
				}
			for (Valve v : getValves(irrigator, "Z3")){
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		// 2a2. AquaLush ... records that
		// this Valve failed in its persistent store; then the use case
		// continues.

		
		List<String> failedDevices = null;
		try {
			failedDevices = getFailedDevices(irrigator);
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		assertEquals(2, failedDevices.size());
		assertTrue(failedDevices.contains("V06"));
		assertTrue(failedDevices.contains("V07"));

		try {
			assertEquals("false", store.getData("V06.isWorking", "true"));
			assertEquals("false", store.getData("V07.isWorking", "true"));
		} catch (StoreFailureException e) {
			e.printStackTrace();
			fail();
		}

	}

	@Test
	public void testStepThree() {

		// 2a2. AquaLush alerts the Operator of the failure ... ; then the use
		// case continues.
		
		// Wait until AquaLush updates its state.
		clock.tick();
		
		try {
			assertTrue(extractField(uiController, "crntScrn") == extractField(
					uiController, "deviceFailureScrn"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// 3. AquaLush records the mode as automatic, ...

		try {
			assertEquals(Mode.AUTOMATIC.toString(), store.getData("mode", "manual"));
		} catch (StoreFailureException e) {
			e.printStackTrace();
			fail();
		}

		// ... and waits for the next irrigation time.

		clock.tick();

		try {
			assertNull(extractField(irrigator, "cycle"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

	}

}
