package irrigation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JCheckBox;

import device.StoreFailureException;
import static tools.HelperFunctions.*;
import static irrigation.ScenarioHelper.*;
import static org.junit.Assert.*;

/**
 * Tests extension 4a2a of the seventh use case.
 */
public class UseCaseSevenExtension4a2aTest extends UseCaseSevenExtension4aTest {

	@Override
	public void testIrrigation() {
		// For each active zone:

		JCheckBox s2 = null;
		JCheckBox v06 = null;

		Zone currentZone;
		IrrigationCycle currentCycle;
		int zoneCount = 0;

		for (JCheckBox cb : checkboxes) {
			if (cb.getText().equals("S2"))
				s2 = cb;
			if (cb.getText().equals("V06"))
				v06 = cb;
		}
		
		HashMap<String, Integer> workingValves = new HashMap<String, Integer>();
		workingValves.put("Z1", 4);
		workingValves.put("Z2", 4);
		workingValves.put("Z3", 4);

		LinkedList<String> activeZones = new LinkedList<String>();
		activeZones.add("Z1");
		activeZones.add("Z2");
		activeZones.add("Z3");
		
		int waterUsed = 0;

		try {
			while ((currentCycle = extractField(irrigator, "cycle")) != null
					&& (currentZone = extractField(currentCycle, "crntZone")) != null) {

				zoneCount++;
				
				// 5b The zone allocation is not exhausted:
				// 5b1. AquaLush adds
				// the remainder of the zone allocation to the zone allocations
				// of the unirrigated active zones and divides this sum by the
				// number of Valves in the unirrigated active zones to compute a
				// new Valve allocation.
				// 5b2. AquaLush recomputes the zone
				// allocations for each active unirrigated zone by multiplying
				// the new Valve allocation by the number of Valves in the zone.

				int numberOfValves = 0;
				for (String zone : activeZones) {
					numberOfValves += workingValves.get(zone);
				}

				assertEquals(
						(int) (((double) (600 - waterUsed) / numberOfValves) * workingValves
								.get(currentZone.getID())),
						extractField(currentZone, "allocation"));

				// 3. AquaLush opens all the (working) Valves in the zone, and
				// marks the time when it does so.
				for (Valve v : getValves(irrigator, currentZone.getID()))
					assertTrue(v.isOpen());

				if (currentZone.getID().equals("Z2")) {
					for (int i = 0; i < 5; i++)
						clock.tick();

					// Let sensor S2 and valve V06 fail.
					
					s2.doClick();
					v06.doClick();

					clock.tick();

					List<String> failedDevices = null;
					try {
						failedDevices = getFailedDevices(irrigator);
					} catch (Exception e) {
						e.printStackTrace();
						fail();
					}
					
					assertTrue(failedDevices.contains("V06"));
					assertTrue(failedDevices.contains("S2"));

				} else {
					int availableAllocation = extractField(currentZone,
							"allocation");
					int usedAllocation = currentZone.getWaterUsed();
					while (currentZone.getWaterUsed() < availableAllocation) {
						clock.tick();

						// 4. Every minute, AquaLush reads the zone Sensor and
						// also computes the
						// zone water usage by multiplying the total flow rate
						// for all Valves in
						// the zone by the time that the Valves have been open.
						assertTrue(usedAllocation < currentZone.getWaterUsed());
						usedAllocation = currentZone.getWaterUsed();
					}
				}

				// 5. If the zones critical moisture level is reached, or the
				// zone allocation is reached, then AquaLush closes all the
				// Valves in the zone.
				// 4a2. AquaLush closes all Valves in the zone.
				for (Valve v : getValves(irrigator, currentZone.getID()))
					if (v.getID().equals("V06")){ // the failed valve stays open
						assertTrue(v.isOpen());
					} else{
						assertFalse(v.isOpen());
					}
				
				activeZones.remove(currentZone.getID());
				waterUsed += currentZone.getWaterUsed();
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		assertEquals(3, zoneCount);

		for (Zone z : zones) {
			assertTrue(irrigator.getWaterUsed() < 624);
			try {
				for (Valve v : getValves(irrigator, z.getID()))
					if (v.getID().equals("V06"))
						assertTrue(z.getID() + ":" + v.getID(), v.isOpen());
					else
						assertFalse(z.getID() + ":" + v.getID(), v.isOpen());
			} catch (Exception e) {
				e.printStackTrace();
				fail();
			}
		}

		// 4a3. AquaLush records that this Sensor failed in its persistent store.

		try {
			assertEquals("false", store.getData("S2.isWorking", "true"));
			assertEquals("false", store.getData("V06.isWorking", "true"));
		} catch (StoreFailureException e) {
			fail();
		}
	}
}
