package irrigation;

import static tools.HelperFunctions.extractField;

import java.awt.Component;
import java.io.Reader;
import java.io.StringReader;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.junit.Before;

import simulation.SimTime;
import simulation.Simulation;
import startup.StartupScenarioHelper;
import ui.UIController;
import device.Clock;
import device.DeviceFactory;
import device.StorageDevice;
import device.sim.SimDeviceFactory;

/**
 * Super class of all tests testing one of AquaLush's use cases.
 */
public abstract class UseCaseTest {

	protected final String configuration = "zone Z1 <Western plot>" + "{"
			+ "sensor S1 <Middle of western plot>;"
			+ "valve V01 <j2342> 2 <NW quadrant of zone Z1>;"
			+ "valve V02 <j2344> 4 <NE quadrant of zone Z1>;"
			+ "valve V03 <j2344> 4 <SW quadrant of zone Z1>;"
			+ "valve V04 <j2344> 4 <SE quadrant of zone Z1>;" + "}"
			+ "zone Z2 <Center plot>" + "{"
			+ "sensor S2 <Middle of center plot>;"
			+ "valve V05 <j2342> 2 <NW quadrant of zone Z2>;"
			+ "valve V06 <j2346> 6 <NE quadrant of zone Z2>;"
			+ "valve V07 <j2342> 2 <SW quadrant of zone Z2>;"
			+ "valve V08 <j2342> 2 <SE quadrant of zone Z2>;" + "}"
			+ "zone Z3 <Eastern plot>" + "{"
			+ "sensor S3 <Middle of eastern plot>;"
			+ "valve V09 <j2346> 6 <NW quadrant of zone Z3>;"
			+ "valve V10 <j2346> 6 <NE quadrant of zone Z3>;"
			+ "valve V11 <j2346> 6 <SW quadrant of zone Z3>;"
			+ "valve V12 <j2346> 6 <SE quadrant of zone Z3>;" + "}";

	protected Simulation simulation;
	protected Irrigator irrigator;
	protected UIController uiController;
	protected JButton[] buttons;
	protected JButton[] keys;
	protected JFrame frame;
	protected StorageDevice store;
	protected Clock clock;
	protected Set<Zone> zones;
	protected Set<JCheckBox> checkboxes;

	/**
	 * Set up the application and extract interesting data.
	 */
	@Before
	public void init() throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException, InterruptedException {
		simulation = new Simulation(null);
		DeviceFactory deviceFactory = new SimDeviceFactory(simulation);
		Reader configSource = new StringReader(configuration);

		// Extract the irrigator and the uiController
		Object[] objects = StartupScenarioHelper.getIrrigatorUIControler(
				deviceFactory, configSource);
		irrigator = (Irrigator) objects[0];
		uiController = (UIController) objects[1];

		// Get the store and a collection of all zones
		store = extractField(irrigator, "store");
		zones = extractField(irrigator, "zones");
		
		// Finish the configuration of the simulation
		// by setting the irrigator
		simulation.setIrrigator(irrigator);
		SimTime.instance().stop();
		clock = Clock.instance();
		
		// Extract the buttons to the left and to the right of the display.
		buttons = extractField(
				extractField(extractField(simulation, "controlPanel"),
						"display"), "scrnBtn");
		
		// Extract the keys of the control panel
		keys = extractField(
				extractField(extractField(simulation, "controlPanel"), "keypad"),
				"key");

		// Extract the checkboxes to change the failure status of
		// a valve or a sensor.
		Component[] comps = ((JPanel) extractField(
				extractField(simulation, "environment"), "failControl"))
				.getComponents();
		JPanel innerPanel = null;
		for (int i=0; i<comps.length; i++)
			if (comps[i] instanceof JPanel)
				innerPanel = (JPanel) comps[i];
		comps = innerPanel.getComponents();
		checkboxes = new HashSet<JCheckBox>();
		for (int i = 0; i < comps.length; i++)
			if (comps[i] instanceof JCheckBox)
				checkboxes.add((JCheckBox) comps[i]);

		// Add the simulation to a frame.
		frame = new JFrame();
		frame.add(simulation);
		frame.pack();
		frame.setVisible(true);
	}
}
