package irrigation;

import static irrigation.ScenarioHelper.*;
import static org.junit.Assert.*;

import org.junit.Test;

/**
 * Tests the basic flow of AquaLush's third use case.
 */
public class UseCaseThreeBasicFlowTest extends UseCaseTest {

	public void stepOne() {
		// 1. Operator selects a non-empty set of closed Valves and directs that
		// they be opened.

		// Go to the "Control Irrigation" screen, ...
		buttons[7].doClick();
		// ... select valve "V05", ...
		for (int i = 0; i < 3; i++)
			buttons[3].doClick();
		// ... open the valve ...
		buttons[2].doClick();
		// ... and propagate the change to the whole zone.
		buttons[4].doClick();

	}

	public void stepThree() {
		// 5. Operator selects a set of closed Valves and directs that they be
		// opened.

		// Scroll one valve down ...
		buttons[5].doClick();
		// ... open the valve ...
		buttons[2].doClick();
		// ... and propagate the change to the whole zone.
		buttons[4].doClick();

	}

	public void stepFive() {
		// 3. Operator selects a set of open Valves and directs that they be
		// closed.

		// Scroll two valves up ...
		buttons[3].doClick();
		buttons[3].doClick();
		// ... close the valve ...
		buttons[2].doClick();
		// ... and propagate the change to the whole zone.
		buttons[4].doClick();

	}

	public void stepSeven() {
		// 7. The Operator indicates that he or she is finished.

		// Leave the "Control Irrigation" screen.
		buttons[7].doClick();

	}

	public void prepareUseCase() {
		// Change the current mode from automatic to manual

		buttons[3].doClick();
		
		assertEquals(0, irrigator.getWaterUsed());

	}

	public void testStepTwo() {
		// 2. AquaLush opens each Valve in the set and displays to the Operator
		// for each open Valve: its location, how long it has been open, how
		// much water it has used, and the moisture level of its associated
		// Sensor. AquaLush also shows the total water used since the start of
		// the use case.
		
		assertEquals(0, irrigator.getWaterUsed());

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				assertTrue(v.isOpen());
				assertTrue(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 3; i++)
			clock.tick();

		assertEquals(36, irrigator.getWaterUsed());

	}

	public void testStepFour() {
		// 6. AquaLush opens the selected Valves and adds them to the Valve
		// status display.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertTrue(v.isOpen());
				assertTrue(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				assertTrue(v.isOpen());
				assertTrue(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 3; i++)
			clock.tick();

		assertEquals(114, irrigator.getWaterUsed());
	}

	public void testStepSix() {
		// 4. AquaLush closes the indicated Valves and removes them from the
		// Valve status display.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertTrue(v.isOpen());
				assertTrue(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 3; i++)
			clock.tick();

		assertEquals(156, irrigator.getWaterUsed());
	}

	public void testStepEight() {
		// 8. AquaLush acknowledges that manual irrigation is finished and
		// closes all Valves.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 3; i++)
			clock.tick();

		assertEquals(0, irrigator.getWaterUsed());
	}

	@Test
	public void executeUseCase() {
		prepareUseCase();
		stepOne();
		testStepTwo();
		stepThree();
		testStepFour();
		stepFive();
		testStepSix();
		stepSeven();
		testStepEight();
	}

}
