package irrigation;

import static irrigation.ScenarioHelper.getValves;
import static org.junit.Assert.*;
import static tools.HelperFunctions.*;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JCheckBox;

import device.StoreFailureException;

/**
 * Tests extension 2a of the third use case.
 */
public class UseCaseThreeExtension2aTest extends UseCaseThreeBasicFlowTest {

	JCheckBox v06;
	ArrayList<String> failedDevices = new ArrayList<String>();

	@Override
	public void stepOne() {
		// Suddenly a valve fails.

		for (JCheckBox cb : checkboxes)
			if (cb.getText().equals("V06"))
				v06 = cb;

		v06.doClick();

		super.stepOne();
	}

	@Override
	public void stepThree() {
		//v06.doClick();
		// quit the failure screen
		buttons[7].doClick();
		// ... select valve "V05", ...
		for (int i = 0; i < 3; i++)
			buttons[3].doClick();
		super.stepThree();
	}

	@Override
	public void testStepTwo() {
		// 2a2. AquaLush alerts the Operator of the failure and records that
		// this Valve failed in its persistent store; then the use case
		// continues.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				if (v.getID().equals("V06")) {
					assertFalse(v.isOpen());
					assertFalse(simulation.getValve(v.getID()).isOpen());
				} else {
					assertTrue(v.isOpen());
					assertTrue(simulation.getValve(v.getID()).isOpen());
				}
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		//String currentFailure;
		try {
			failedDevices = extractField(irrigator, "failedDevices");
		} catch (Exception e2) {
		}
		assertTrue(failedDevices.contains("V06"));
		
		// Wait until AquaLush updates its state.
		clock.tick();
		
		try {
			assertTrue(extractField(uiController, "crntScrn") == extractField(
					uiController, "deviceFailureScrn"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		try {
			assertEquals("false", store.getData("V06.isWorking", "true"));
		} catch (StoreFailureException e1) {
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 2; i++)
			clock.tick();

		assertEquals(18, irrigator.getWaterUsed());
		
	}
	
	@Override
	public void testStepFour() {
		// 6. AquaLush opens the selected Valves and adds them to the Valve
		// status display.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertTrue(v.isOpen());
				assertTrue(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				if (v.getID().equals("V06")) {
					assertFalse(v.isOpen());
					assertFalse(simulation.getValve(v.getID()).isOpen());
				} else {
					assertTrue(v.isOpen());
					assertTrue(simulation.getValve(v.getID()).isOpen());
				}
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 3; i++)
			clock.tick();

		assertEquals(78, irrigator.getWaterUsed());
	}
	
	@Override
	public void testStepSix() {
		// 4. AquaLush closes the indicated Valves and removes them from the
		// Valve status display.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertTrue(v.isOpen());
				assertTrue(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 3; i++)
			clock.tick();

		assertEquals(120, irrigator.getWaterUsed());
	}
}
