package irrigation;

import static irrigation.ScenarioHelper.getFailedDevices;
import static irrigation.ScenarioHelper.getValves;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static tools.HelperFunctions.extractField;

import java.util.List;

import javax.swing.JCheckBox;

import device.StoreFailureException;

/**
 * Tests extensions 2a and 6b of the third use case.
 */
public class UseCaseThreeExtension6a2bTest extends UseCaseThreeExtension6aTest {

	JCheckBox s2;

	@Override
	public void stepOne() {

		// Suddenly, the second zone's sensor fails.
		for (JCheckBox cb : checkboxes)
			if (cb.getText().equals("S2"))
				s2 = cb;

		s2.doClick();

		super.stepOne();
	}

	@Override
	public void stepThree() {
		// quit the failure screen
		buttons[7].doClick();
		// ... select valve "V05", ...
		for (int i = 0; i < 3; i++)
			buttons[3].doClick();
			
		// Let the failed sensor work again.
		
		s2.doClick();
		super.stepThree();
	}

	@Override
	public void testStepTwo() {
		// 2b2. AquaLush alerts the Operator of the failure and records that
		// this Sensor failed in its persistent store; then the use case
		// continues.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				assertTrue(v.isOpen());
				assertTrue(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		List<String> failedDevices = null;
		try {
			failedDevices = getFailedDevices(irrigator);
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		assertEquals("S2", failedDevices.get(0));
		
		// Let AquaLush update its state.
		clock.tick();

		try {
			assertTrue(extractField(uiController, "crntScrn") == extractField(
					uiController, "deviceFailureScrn"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		try {
			assertEquals("false", store.getData("S2.isWorking", "true"));
		} catch (StoreFailureException e1) {
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 2; i++)
			clock.tick();

		assertEquals(36, irrigator.getWaterUsed());

	}

}
