package irrigation;

import static irrigation.ScenarioHelper.getFailedDevices;
import static irrigation.ScenarioHelper.getValves;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static tools.HelperFunctions.extractField;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JCheckBox;

import device.StoreFailureException;

/**
 * Tests extension 6a of the third use case.
 */
public class UseCaseThreeExtension6aTest extends UseCaseThreeBasicFlowTest {

	private JCheckBox v05;
	protected Set<String> failedDevices = new HashSet<String>();

	@Override
	public void stepFive() {
		// Suddenly a valve fails.

		for (JCheckBox cb : checkboxes)
			if (cb.getText().equals("V05"))
				v05 = cb;

		v05.doClick();

		super.stepFive();
	}

	@Override
	public void testStepSix() {
		// 2a2. AquaLush alerts the Operator of the failure and records that
		// this Valve failed in its persistent store; then the use case
		// continues.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertTrue(v.isOpen());
				assertTrue(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				if (v.getID().equals("V05")) {
					assertTrue(v.isOpen());
					assertTrue(simulation.getValve(v.getID()).isOpen());
				} else {
					assertFalse(v.isOpen());
					assertFalse(simulation.getValve(v.getID()).isOpen());
				}
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		List<String> failedDevices = null;
		try {
			failedDevices = getFailedDevices(irrigator);
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		assertTrue(failedDevices.contains("V05"));
		
		// Let AquaLush update its state.
		clock.tick();

		try {
			assertTrue(extractField(uiController, "crntScrn") == extractField(
					uiController, "deviceFailureScrn"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		try {
			assertEquals("false", store.getData("V05.isWorking", "true"));
		} catch (StoreFailureException e1) {
			fail();
		}

		// quit the devide failure screen.
		buttons[7].doClick();
		
		// A few minutes pass.
		for (int i = 0; i < 2; i++)
			clock.tick();

		assertEquals(144, irrigator.getWaterUsed());

	}
	
	@Override
	public void testStepEight() {
		// 8. AquaLush acknowledges that manual irrigation is finished and
		// closes all Valves.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				if (v.getID().equals("V05")) {
					assertTrue(v.isOpen());
					assertTrue(simulation.getValve(v.getID()).isOpen());
				} else {
					assertFalse(v.isOpen());
					assertFalse(simulation.getValve(v.getID()).isOpen());
				}
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// A few minutes pass.
		for (int i = 0; i < 3; i++)
			clock.tick();

		assertEquals(0, irrigator.getWaterUsed());
	}

}
