package irrigation;

import static irrigation.ScenarioHelper.getFailedDevices;
import static irrigation.ScenarioHelper.getValves;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static tools.HelperFunctions.extractField;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.AbstractButton;
import javax.swing.JCheckBox;

import device.StoreFailureException;

/**
 * Tests extension 8a of the third use case.
 */
public class UseCaseThreeExtension8aTest extends UseCaseThreeBasicFlowTest {

	private AbstractButton v02;
	private Set<String> failedDevices = new HashSet<String>();

	@Override
	public void stepSeven() {

		// Suddenly a valve fails.

		for (JCheckBox cb : checkboxes)
			if (cb.getText().equals("V02"))
				v02 = cb;

		v02.doClick();

		super.stepSeven();
	}

	@Override
	public void testStepEight() {
		// 2a2. AquaLush alerts the Operator of the failure and records that
		// this Valve failed in its persistent store; then the use case
		// continues.

		try {
			for (Valve v : getValves(irrigator, "Z1")) {
				if (v.getID().equals("V02")) {
					assertTrue(v.isOpen());
					assertTrue(simulation.getValve(v.getID()).isOpen());
				} else {
					assertFalse(v.isOpen());
					assertFalse(simulation.getValve(v.getID()).isOpen());
				}
			}
			for (Valve v : getValves(irrigator, "Z2")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
			for (Valve v : getValves(irrigator, "Z3")) {
				assertFalse(v.isOpen());
				assertFalse(simulation.getValve(v.getID()).isOpen());
			}
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		List<String> failedDevices = null;
		try {
			failedDevices = getFailedDevices(irrigator);
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		assertTrue(failedDevices.contains("V02"));
		
		// Let AquaLush update its state.
		clock.tick();
		
		try {
			assertTrue(extractField(uiController, "crntScrn") == extractField(
					uiController, "deviceFailureScrn"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
		
		try {
			assertEquals("false", store.getData("V02.isWorking", "true"));
		} catch (StoreFailureException e1) {
			fail();
		}
	}

}
