package irrigation;

import simulation.SimTime;
import device.StoreFailureException;
import static junit.framework.Assert.*;

/**
 * Tests the basic flow of AquaLush's second use case by manipulating
 * the critical moisture level.
 */
public class UseCaseTwoTestBasicFlowCriticalLevel extends UseCaseTwoTest {

	@Override
	protected void stepOne() {
		// 1. Operator sets the irrigation time, water allocation, or critical
		// moisture levels.

		// Open "Set up Automatic Irrigation", ...
		buttons[4].doClick();
		// ... select "Set Critical Moisture Level" ...
		buttons[5].doClick();
		//
		// ... and change the level to 30.
		for (int i = 0; i < 2; i++)
			keys[9].doClick();

		keys[2].doClick();
		keys[10].doClick();

		// Propagate this change to all zones.
		buttons[2].doClick();

		// Accept the settings
		buttons[7].doClick();
	}

	@Override
	public void testStepOne() {
		// 3. AquaLush records and confirms the new setting(s).

		for (ZoneReport r : irrigator.getZoneReports()) {
			assertEquals(30, r.criticalLevel);
			try {
				assertEquals("30",
						store.getData(r.id + ".criticalMoistureLevel", "50"));
			} catch (StoreFailureException e) {
				e.printStackTrace();
				fail();
			}
		}

	}

	@Override
	public void testStepTwo() {
	}

	@Override
	public void testStepThree() {
	}

	@Override
	public void prepareUseCase() {
		SimTime time = SimTime.instance();
		time.setMinute(0);
		time.setSecond(0);
		time.setHour(10);
	}

}
