package irrigation;

import device.StoreFailureException;
import simulation.SimTime;
import util.Day;
import static junit.framework.Assert.*;

/**
 * Tests the basic flow of AquaLush's second use case by manipulating
 * the irrigation time.
 */
public class UseCaseTwoTestBasicFlowIrrigationTime extends UseCaseTwoTest {

	@Override
	protected void stepOne() {
		// 1. Operator sets the irrigation time, water allocation, or critical
		// moisture levels.

		// Open "Set up Automatic Irrigation", ...
		buttons[4].doClick();
		// ... select "Set Irrigation Time", ...
		buttons[3].doClick();
		// ... deselect all days except Friday ...
		for (int i = 2; i < 7; i++)
			buttons[i].doClick();
		// ... and change the irrigation time to 1200.
		for (int i = 0; i < 4; i++)
			keys[9].doClick();

		keys[0].doClick();
		keys[1].doClick();
		keys[10].doClick();
		keys[10].doClick();

		// Accept the settings
		buttons[7].doClick();
	}

	@Override
	public void testStepOne() {
		// 3. AquaLush records and confirms the new setting(s).

		for (int i = 0; i < 6; i++)
			if (i == 4 || i == 0)
				assertTrue(irrigator.getIrrigationDays().contains(
						Day.valueOf(i)));
			else
				assertFalse(irrigator.getIrrigationDays().contains(
						Day.valueOf(i)));

		assertEquals(1200, irrigator.getIrrigationTime());

		String irrigationDays;
		try {
			irrigationDays = store.getData("irrigationDays", "SaSo");

			assertTrue(irrigationDays.equals("MF")
					|| irrigationDays.equals("FM"));
			assertEquals("1200", store.getData("irrigationTime", "0200"));
		} catch (StoreFailureException e) {
			e.printStackTrace();
			fail();
		}

	}

	@Override
	public void testStepTwo() {

	}

	@Override
	public void testStepThree() {

	}

	@Override
	public void prepareUseCase() {
		SimTime time = SimTime.instance();
		time.setMinute(0);
		time.setSecond(0);
		time.setHour(10);
	}

}
