package irrigation;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;
import static tools.HelperFunctions.extractField;
import simulation.SimTime;

/**
 * Tests extension 3d to the basic flow of the
 * first use case.
 */
public class UseCaseTwoTestRunningCycleCriticalLevel extends
		UseCaseTwoTestBasicFlowCriticalLevel {

	@Override
	public void prepareUseCase() {
		// Set the clock so that an automatic irrigation
		// cycle is started soon.
		SimTime time = SimTime.instance();
		time.setMinute(0);
		time.setSecond(0);
		time.setHour(2);

		// Let a few minutes pass.
		for (int i = 0; i < 7; i++)
			clock.tick();

		// Now, an irrigation cycle is running.
		try {
			assertNotNull(extractField(irrigator, "cycle"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
	}

	@Override
	public void testStepTwo() {
		// 3d1. AquaLush uses the new critical moisture levels to control
		// irrigation immediately.

		// Irrigate for a few minutes, ...
		for (int i = 0; i < 5; i++)
			clock.tick();

		// Since the new critical moisture level was in effect immediately,
		// the irrigation should stop.
		try {
			assertNull(extractField(irrigator, "cycle"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

	}
}
