package irrigation;

import static org.junit.Assert.*;
import static tools.HelperFunctions.extractField;
import simulation.SimSensor;
import simulation.SimTime;
import util.Day;

/**
 * Tests extension 3b to the basic flow of the
 * first use case.
 */
public class UseCaseTwoTestRunningCycleIrrigationTime extends
		UseCaseTwoTestBasicFlowIrrigationTime {

	@Override
	public void prepareUseCase() {
		// Set the clock so that an automatic irrigation
		// cycle is started soon.
		SimTime time = SimTime.instance();
		time.setMinute(0);
		time.setSecond(0);
		time.setHour(2);

		// Let a few minutes pass.
		for (int i = 0; i < 7; i++)
			clock.tick();
	}

	@Override
	public void testStepTwo() {
		// 3d1. AquaLush uses the new critical moisture levels to control
		// irrigation immediately.
		IrrigationCycle cycle;
		Zone currentZone = null;
		try {
			while ((cycle = extractField(irrigator, "cycle")) != null
					&& (currentZone = extractField(cycle, "crntZone")) != null) {

				// Irrigate for a few minutes, ...
				for (int i = 0; i < 3; i++)
					clock.tick();

				// ... then let the current zone reach its new
				// critical moisture level.
				SimSensor currentSensor = simulation.getSensor(currentZone
						.getID().replace("Z", "S"));
				currentSensor.setLevel(31);

				// Change zone if necessary.
				for (int i = 0; i < 2; i++)
					clock.tick();

			}

			// Since all zones have reached their critical moisture level,
			// the irrigation should stop.
			assertTrue(cycle == null || currentZone == null);

		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

		// When the new irrigation time is reached,
		// a fresh irrigation cycle should start.
		SimTime.instance().setDay(Day.FRIDAY);
		SimTime.instance().setHour(12);

		clock.tick();

		try {
			assertNotNull(extractField(irrigator, "cycle"));
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}

	}
}
