package device;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import util.Day;

public abstract class ClockDeviceTest {

	protected abstract ClockDevice getOut();

	protected abstract void tick(ClockDevice c);

	protected static int ticks;
	protected ClockDevice clock;
	
	@Before
	public void init() {
		ticks = 0;
		clock = getOut();
	}

	@Test
	public void testTimeProperty() {
		// Tests setting and querying the
		// current time
		clock.setTime(0000);
		assertEquals(0000, clock.getTime());

		clock.setTime(2359);
		assertEquals(2359, clock.getTime());
	}

	@Test
	public void testTimePropertyIllegalCases() {
		// Tests whether setTime() detects
		// invalid military times
		try {
			clock.setTime(2412);
			fail();
		} catch (IllegalArgumentException e) {

		}

		try {
			clock.setTime(-1);
			fail();
		} catch (IllegalArgumentException e) {

		}

		try {
			clock.setTime(1260);
			fail();
		} catch (IllegalArgumentException e) {

		}
	}

	@Test
	public void testDayProperty() {
		// Tests the getters and setters
		// of day
		clock.setDay(Day.MONDAY);
		assertEquals(Day.MONDAY, clock.getDay());

		clock.setDay(Day.SUNDAY);
		assertEquals(Day.SUNDAY, clock.getDay());
	}

	@Test
	public void testDayPropertyNull() {
		// Tests how setDay behaves if
		// null is passed
		clock.setDay(null);
		assertNull(clock.getDay());
	}

	@Test
	public void testSetListener() {
		// Tests whether setListener() registers
		// a new listener
		TickListener listener1 = new TickListener() {

			@Override
			public void tick() {
				ticks++;
			}
		};

		clock.setListener(listener1);

		tick(clock);

		assertEquals(1, ticks);

		TickListener listener2 = new TickListener() {

			@Override
			public void tick() {
				ticks++;
			}
		};

		clock.setListener(listener2);

		tick(clock);

		// Since only one listener can be registered at any
		// time, only one additional tick is expected
		assertEquals(2, ticks);
	}
	
	@Test
	public void testSetListenerNull() {
		// Tests how tick sets the listener to null
		clock.setListener(null);

		tick(clock);

		assertEquals(0, ticks);

	}

}
