package device;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

public abstract class DeviceFactoryTest {

	protected DeviceFactory factory;

	protected abstract DeviceFactory getOut();

	@Before
	public void init() {
		factory = getOut();
	}

	@Test
	public void testCreateSensorDevice() {
		// Tests createSensorDevice().
		assertNotNull(factory.createSensorDevice("S1"));
	}

	@Test
	public void testCreateSensorDeviceNull() {
		// createSensorDevice() receives null as the argument.
		try {
			factory.createSensorDevice(null);
			fail();
		} catch (IllegalArgumentException e) {

		}
	}

	@Test
	public void testCreateValveDevice() {
		// Tests createValveDevice().
		assertNotNull(factory.createSensorDevice("V1"));
	}
	
	@Test
	public void testCreateValveDeviceNull() {
		// Tests createValveDevice() with the argument null.
		try {
			factory.createSensorDevice(null);
			fail();
		} catch (IllegalArgumentException e) {

		}
	}

	@Test
	public void testCreateDisplayDevice() {
		// Tests createDisplayDevice().
		assertNotNull(factory.createDisplayDevice());
	}

	@Test
	public void testCreateKeypadDevice() {
		// Tests createKeypadDevice().
		assertNotNull(factory.createKeypadDevice());

	}

	@Test
	public void testCreateScreenButtonDevice() {
		// Tests createScreenButtonDevice().
		assertNotNull(factory.createScreenButtonDevice());
	}

	@Test
	public void testCreateClockDevice() {
		// Tests createClockDevice().
		assertNotNull(factory.createClockDevice());
	}

	@Test
	public void testCreateStorageDevice() {
		// Tests createStorageDevice().
		assertNotNull(factory.createStorageDevice());
	}

}
