package device;

import static org.junit.Assert.*;

import java.util.HashSet;

import org.junit.Before;
import org.junit.Test;

public class DeviceFailureExceptionTest {
	
	protected DeviceFailureException e;

	@Before
	public void init() {
		e = new DeviceFailureException();
	}

	@Test
	public void testDeviceFailureException() {
		// A newly created DeviceFailureException should
		// not contain any devices.
		assertTrue(e.getDevices().isEmpty());
	}

	@Test
	public void testDeviceFailureExceptionEmptyCollectionOfStrings() {
		// Tests whether an empty set of Strings is properly
		// processed.
		DeviceFailureException e = new DeviceFailureException(
				new HashSet<String>());
		assertTrue(e.getDevices().isEmpty());
	}

	@Test
	public void testDeviceFailureExceptionCollectionOfStrings() {
		// Tests whether an non-empty set of Strings is properly
		// processed.
		HashSet<String> set = new HashSet<String>();
		set.add("1");
		set.add("2");
		set.add("3");

		// All strings should be contained in the set
		// returned by the exception
		e = new DeviceFailureException(set);
		assertEquals(3, e.getDevices().size());
		for (int i = 1; i <= 3; i++) {
			String s = new Integer(i).toString();
			assertTrue(e.getDevices().contains(s));
			e.getDevices().remove(s);
		}
	}

	@Test
	public void testDeviceFailureExceptionNull() {
		// Tests whether null is properly handled.
		try {
			e = new DeviceFailureException(null);
			fail();
		} catch (NullPointerException e1) {

		}
	}

	@Test
	public void testAddException() {
		// Tests adding an exception containing some stored devices.
		HashSet<String> set = new HashSet<String>();
		set.add("1");
		set.add("2");

		DeviceFailureException e = new DeviceFailureException(set);

		DeviceFailureException e2 = new DeviceFailureException();
		e2.addException(e);
		assertEquals(2, e2.getDevices().size());
		for (int i = 1; i <= 2; i++) {
			String s = new Integer(i).toString();
			assertTrue(e2.getDevices().contains(s));
		}

		// Tests adding an empty exception.
		DeviceFailureException e3 = new DeviceFailureException();
		e2.addException(e3);

		assertEquals(2, e2.getDevices().size());
		for (int i = 1; i <= 2; i++) {
			String s = new Integer(i).toString();
			assertTrue(e2.getDevices().contains(s));
		}
	}

	@Test
	public void testAddExceptionNull() {
		// Tests adding null to an existing exception.
		DeviceFailureException e2 = new DeviceFailureException();
		try {
			e2.addException(null);
			fail();
		} catch (IllegalArgumentException e4) {

		}
	}

	@Test
	public void testAddDevice() {
		// Tests addDevice().
		DeviceFailureException e = new DeviceFailureException();
		e.addDevice("1");
		assertEquals(1, e.getDevices().size());
		assertEquals("1", e.getDevices().toArray()[0]);

		e.addDevice("2");
		assertEquals(2, e.getDevices().size());
		for (int i = 1; i <= 2; i++) {
			String s = new Integer(i).toString();
			assertTrue(e.getDevices().contains(s));
		}
	}

	@Test
	public void testGetDevices() {
		// Tests getDevices().
		HashSet<String> set = new HashSet<String>();
		set.add("a");
		set.add("b");

		DeviceFailureException e = new DeviceFailureException(set);
		assertEquals(set, e.getDevices());
	}
}
