package device;

import static org.junit.Assert.*;

import org.junit.Test;

public abstract class ScreenButtonDeviceTest {

	protected abstract ScreenButtonDevice getOut();

	protected abstract void notifyListener(ScreenButtonDevice dev);

	private static int ticks = 0;

	@Test
	public void testSetListener() {
		// Tests setListener().
		ScreenButtonListener l = new ScreenButtonListener() {

			@Override
			public void screenButtonPress(int whichButton) {
				ticks++;
			}
		};

		ScreenButtonDevice dev = getOut();
		dev.setListener(l);

		// Does the listener receive notifications?
		int oldTicks = ticks;

		notifyListener(dev);

		assertEquals(oldTicks + 1, ticks);
		oldTicks = ticks;
	}

	@Test
	public void testSetListenerNull() {
		// Tests setListener(). Only one listener exists
		// at a given time.
		
		ScreenButtonListener l = new ScreenButtonListener() {

			@Override
			public void screenButtonPress(int whichButton) {
				ticks++;
			}
		};

		int oldTicks = ticks;
		ScreenButtonDevice dev = getOut();
		dev.setListener(l);
		dev.setListener(null);

		notifyListener(dev);

		// Since the current listener is null,
		// ticks should not change.
		assertEquals(oldTicks, ticks);

	}

}
