package device;

import static org.junit.Assert.*;

import org.junit.Test;

public abstract class StorageDeviceTest {
	
	protected abstract StorageDevice getOut();

	@Test
	public void testGetSetData() throws StoreFailureException {
		// Tests getData() and setData().
		StorageDevice device = getOut();
		
		assertNull(device.getData("A", null));
		assertEquals("a", device.getData("A", "a"));
		assertEquals("a", device.getData("A", "b"));
		
		device.setData("A", "a");
		assertEquals("a", device.getData("A", "b"));
		device.setData("A", "c");
		assertEquals("c", device.getData("A", "d"));
		device.setData("B", "B1");
		assertEquals("B1", device.getData("B", "D"));
		assertEquals("c", device.getData("A", "E"));
	}
	
	@Test
	public void testGetSetDataIllegalArguments() throws StoreFailureException {
		// Tests whether getData() and setData() check their
		// preconditions.
		StorageDevice device = getOut();
		try {
			device.setData(null, "A");
			fail();
		} catch (IllegalArgumentException e){
			
		}
		
		try {
			device.setData("A", null);
		} catch (IllegalArgumentException e){
			
		}
		
		try {
			device.getData(null, "A");
			fail();
		} catch (IllegalArgumentException e){
			
		}
		
		assertEquals("c", device.getData("A", "f"));
		
	}

	@Test
	public void testRemoveData() throws StoreFailureException {
		// Tests removeData().
		StorageDevice device = getOut();
		
		device.setData("Test1", "test");
		device.setData("Test", "a");
		device.removeData("Test");
		assertEquals("b1", device.getData("Test", "b1"));
		assertEquals("test", device.getData("Test1", null));
	}
		
	@Test
	public void testRemoveDataIllegalArguments() throws StoreFailureException {
		// Tests whether removeData() checks its preconditions.
		StorageDevice device = getOut();
		
		try {
			device.removeData(null);
			fail();
		} catch (IllegalArgumentException e){
			
		}
		
		device.removeData("TestA");
		
		assertEquals("test", device.getData("Test1", null));
		
	}

}
