package device;

import static org.junit.Assert.*;

import org.junit.Test;

public abstract class ValveDeviceTest {

	protected abstract ValveDevice getOut(boolean isFailed);

	protected abstract boolean isOpen(ValveDevice v)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException;

	protected abstract boolean isClosed(ValveDevice v)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException;

	@Test
	public void testOpen() throws DeviceFailureException,
			IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		// Tests open().
		ValveDevice v = getOut(false);
		v.open();
		assertTrue(isOpen(v));

		v.close();
		v.open();
		assertTrue(isOpen(v));
	}

	@Test
	public void testOpenFailedDevice() {
		// Tests open() on a failed ValveDevice.
		ValveDevice v = getOut(true);
		try {
			v.open();
			fail();
		} catch (DeviceFailureException e) {
		}
	}

	@Test
	public void testClose() throws DeviceFailureException,
			IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		// Tests close().
		ValveDevice v = getOut(false);
		v.close();
		assertTrue(isClosed(v));

		v.open();
		v.close();
		assertTrue(isClosed(v));
	}

	@Test
	public void testCloseFailedDevice() {
		// Tests close() on a failed device.
		ValveDevice v = getOut(true);
		try {
			v.close();
			fail();
		} catch (DeviceFailureException e) {
		}
	}

}
