package device.sim;

import static org.junit.Assert.*;
import static tools.HelperFunctions.*;

import org.junit.Test;

import simulation.SimTime;

import device.ClockDevice;
import device.ClockDeviceTest;
import device.TickListener;

public class SimClockDeviceTest extends ClockDeviceTest {

	@Test
	public void testSimClockDevice() throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Tests how a new SimClockDevice() is initialized.
		SimClockDevice c = (SimClockDevice) getOut();
		assertNull(extractField(c, "listener"));
		assertEquals(SimTime.instance(), extractField(c, "time"));

		assertObserved(SimTime.instance(), c);
	}

	@Test
	public void testUpdate() {
		// Tests update().
		SimClockDevice c = (SimClockDevice) getOut();

		SimTime.instance().setSecond(0);
		c.setListener(new TickListener() {

			@Override
			public void tick() {
				ticks++;
			}
		});
		
		c.update(null, null);

		// Since a minute has passed, the listener
		// should be called.
		assertEquals(1, ticks);

		// Since not a full second has passed,
		// the listener should not be called.
		SimTime.instance().setSecond(10);
		c.update(null, null);

		assertEquals(1, ticks);

	}

	@Override
	protected ClockDevice getOut() {
		SimTime.instance().deleteObservers();
		return new SimClockDevice();
	}

	@Override
	protected void tick(ClockDevice c) {
		// Causes the SimTime object to tick
		// by setting its seconds to 0 (causes tick) and then
		// back to their initial value.
		SimTime time = SimTime.instance();
		
		int oldSec = time.getSecond();
		time.setSecond(0);
		
		if (oldSec != 0)
			time.setSecond(oldSec);

	}

}
