package device.sim;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;

import simulation.Simulation;
import device.DeviceFactory;
import device.DeviceFactoryTest;
import device.ScreenButtonDevice;

public class SimDeviceFactoryTest extends DeviceFactoryTest {

	@Override
	public void testCreateSensorDevice() {
		super.testCreateSensorDevice();

		assertTrue(getOut().createSensorDevice("S1") instanceof SimSensorDevice);
	}

	@Override
	public void testCreateValveDevice() {
		super.testCreateValveDevice();

		assertTrue(getOut().createValveDevice("V1") instanceof SimValveDevice);
	}

	@Override
	public void testCreateDisplayDevice() {
		super.testCreateDisplayDevice();

		assertTrue(getOut().createDisplayDevice() instanceof SimDisplayDevice);
	}

	@Override
	public void testCreateKeypadDevice() {
		super.testCreateKeypadDevice();

		assertTrue(getOut().createKeypadDevice() instanceof SimKeypadDevice);
	}

	@Override
	public void testCreateScreenButtonDevice() {
		super.testCreateScreenButtonDevice();

		assertTrue(getOut().createScreenButtonDevice() instanceof ScreenButtonDevice);
	}

	@Override
	public void testCreateClockDevice() {
		super.testCreateClockDevice();

		assertTrue(getOut().createClockDevice() instanceof SimClockDevice);
	}

	@Override
	public void testCreateStorageDevice() {
		super.testCreateStorageDevice();

		assertTrue(getOut().createStorageDevice() instanceof SimStorageDevice);
	}

	@Test
	public void testSimDeviceFactory() {
		// Tests how SimDeviceFactory's constructor checks
		// its precondition.
		try {
			getConcreteOut(new Object[] { null });
			fail();
		} catch (IllegalArgumentException e) {

		}
	}

	@Override
	protected DeviceFactory getOut() {
		// Create the current object under test.
		return getConcreteOut(new Object[] { new Simulation(null) });

	}

	protected SimDeviceFactory getConcreteOut(Object[] args) {
		return new SimDeviceFactory((Simulation) args[0]);
	}

}
