package device.sim;

import static org.junit.Assert.*;

import org.junit.Test;

import simulation.MockSimDisplay;
import simulation.SimDisplay;
import static tools.HelperFunctions.*;

import device.DisplayDevice;
import device.DisplayDeviceTest;

public class SimDisplayDeviceTest extends DisplayDeviceTest {

	private final String BLANK = "                                        ";

	protected SimDisplayDevice getConcreteOut(Object[] args) {
		return new SimDisplayDevice((SimDisplay) args[0]);
	}

	protected MockSimDisplay extractDisplay(SimDisplayDevice dev)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		// Extracts a MockSimDisplay from a SimDisplayDevice.
		return extractField(dev, "display");
	}

	@Test
	public void testSimDisplayDevice() throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Tests a newly created SimDisplayDevice.
		MockSimDisplay mock = new MockSimDisplay(DisplayDevice.NUM_LINES,
				DisplayDevice.NUM_COLS);
		SimDisplayDevice device = getConcreteOut(new Object[] { mock });

		assertEquals(mock, extractField(device, "display"));
	}

	@Test
	public void testSimDisplayDeviceIllegalArguments() {
		// Tests whether exceptions are thrown
		// if a SimDisplayDevice is initialized
		// with null.

		try {
			new SimDisplayDevice(null);
			fail();
		} catch (IllegalArgumentException e) {

		}

	}

	@Override
	protected DisplayDevice getOut() {
		return getConcreteOut(new Object[] { new MockSimDisplay(
				DisplayDevice.NUM_LINES, DisplayDevice.NUM_COLS) });
	}

	@Override
	protected boolean displayCleared(DisplayDevice dev) {
		// Determines whether a DisplayDevice was cleared
		// or not.
		
		MockSimDisplay display = null;

		try {
			display = extractDisplay((SimDisplayDevice) dev);
		} catch (Exception e) {
			fail();
		}

		for (int i = 0; i < DisplayDevice.NUM_LINES; i++)
			if (! display.getLine(i).equals(BLANK))
				return false;

		return true;

	}

	@Override
	protected boolean notHighlighted(DisplayDevice dev) {
		// Determines whether no text on a DisplayDevice
		// is highlighted.
		
		MockSimDisplay display = null;

		try {
			display = extractDisplay((SimDisplayDevice) dev);
		} catch (Exception e) {
			fail();
		}

		for (int i = 0; i < DisplayDevice.NUM_LINES; i++)
			if (display.getLine(i).matches("<html>"))
				return false;

		return true;
	}

	@Override
	protected boolean lineCleared(DisplayDevice dev, int line) {
		// Checks whether a line was cleared on a DisplayDevice.
		
		MockSimDisplay display = null;

		try {
			display = extractDisplay((SimDisplayDevice) dev);
		} catch (Exception e) {
			fail();
		}

		return display.getLine(line).equals(BLANK);
	}

	@Override
	protected boolean isHighlighted(DisplayDevice dev, int line, int start,
			int end) {
		// Checks whether a certain part of a certain line
		// of a DisplayDevice is highlighted or not.
		MockSimDisplay display = null;

		try {
			display = extractDisplay((SimDisplayDevice) dev);
		} catch (Exception e) {
			fail();
		}

		String queriedLine = display.getLine(line);

		String[] htmlSplit = queriedLine.split("<html>");

		if (htmlSplit.length == 1)
			return false;

		String[] fontTagSplit = htmlSplit[1].split("<(/)*font[^>]*>");
		if (fontTagSplit.length < 2)
			return false;
		
		fontTagSplit[1] = fontTagSplit[1].replaceAll("&nbsp;", " ");
		
		return (fontTagSplit[0].length() == start) && (fontTagSplit[1].length() == end);

	}

	@Override
	protected String lineContent(DisplayDevice dev, int line) {
		// Returns the line content of a line.
		
		MockSimDisplay display = null;

		try {
			display = extractDisplay((SimDisplayDevice) dev);
		} catch (Exception e) {
			fail();
		}
		
		return display.getLine(line);
	}

}
