package device.sim;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.awt.event.ActionEvent;

import org.junit.Test;

import tools.HelperFunctions;
import device.KeyPress;
import device.KeypadDevice;
import device.KeypadDeviceTest;
import device.KeypadListener;

public class SimKeypadDeviceTest extends KeypadDeviceTest {

	// Mock keypad listener.
	protected class MockKeypadListener implements KeypadListener {

		public KeyPress lastKeypress = null;

		@Override
		public void keyPress(KeyPress key) {
			lastKeypress = key;
		}

	}

	@Test
	public void testActionPerformed() {
		// Tests actionPerformed().
		SimKeypadDevice device = (SimKeypadDevice) getOut();
		MockKeypadListener l = new MockKeypadListener();

		device.setListener(l);

		device.actionPerformed(new ActionEvent(this, 0, "DEL"));
		assertEquals(KeyPress.DEL_KEY, l.lastKeypress);

		device.actionPerformed(new ActionEvent(this, 0, "ESC"));
		assertEquals(KeyPress.ESC_KEY, l.lastKeypress);

		device.actionPerformed(new ActionEvent(this, 0, "0"));
		assertEquals(KeyPress.ZERO_KEY, l.lastKeypress);

		device.actionPerformed(new ActionEvent(this, 0, "9"));
		assertEquals(KeyPress.NINE_KEY, l.lastKeypress);
	}

	@Test
	public void testActionPerformedIllegalArguments() {
		// Tests whether actionPerformed checks
		// its preconditions.

		SimKeypadDevice device = (SimKeypadDevice) getOut();
		try {
			device.actionPerformed(null);
			fail();
		} catch (IllegalArgumentException e) {

		}

		device.setListener(null);
		device.actionPerformed(new ActionEvent(this, 0, "DEL"));

	}

	@Override
	protected KeypadDevice getOut() {
		return new SimKeypadDevice();
	}

	@Override
	protected boolean keypadListenerIsSet(KeypadListener l, KeypadDevice k)
			throws IllegalArgumentException, SecurityException,
			IllegalAccessException, NoSuchFieldException {
		// Extracts the keypad listener of a KeypadDevice.

		Object listener = null;

		listener = HelperFunctions.extractField(k, "listener");

		if (l == null && listener == null)
			return true;
		else
			return l.equals(listener);
	}

}
