package device.sim;

import org.junit.Test;

import simulation.SimValve;
import tools.HelperFunctions;
import device.DeviceFailureException;
import device.ValveDevice;
import device.ValveDeviceTest;

public class SimValveDeviceTest extends ValveDeviceTest {
	
	protected SimValveDevice getConcreteOut(Object[] args) {
		return new SimValveDevice((SimValve) args[0]);
	}

	@Test
	public void testConstructor() throws DeviceFailureException {
		// Tests the creation of a SimValveDevice. No concrete SimValve
		// is passed to the constructor.
		SimValveDevice v = getConcreteOut(new Object[] {null});
		// Should work because no preconditions are
		// declared for SimValveDevice's constructor
		v.open();
	}

	@Override
	protected ValveDevice getOut(boolean isFailed) {
		SimValve v = new SimValve("V1", 10);
		if (isFailed)
			v.setIsFailed();
		return getConcreteOut(new Object[] {v});
	}

	@Override
	protected boolean isOpen(ValveDevice v) throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Checks whether a ValveDevice is open.
		SimValveDevice simV = (SimValveDevice) v;
		SimValve innerValve = HelperFunctions.extractField(simV, "valve");
		return innerValve.isOpen();
	}

	@Override
	protected boolean isClosed(ValveDevice v) throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Checks whether a ValveDevice is closed.
		return !isOpen(v);
	}

}
