package irrigation;

import static org.junit.Assert.*;

import java.util.Collection;
import java.util.LinkedList;

import org.junit.Before;
import org.junit.Test;

public class FailureReportTest {

	private Collection<SensorReport> sensorReports;
	private Collection<ValveReport> valveReports;

	@Before
	public void init() {
		// Set up the collections of sensor and valve
		// reports.
		sensorReports = new LinkedList<SensorReport>();
		for (int i = 0; i < 2; i++) {
			sensorReports.add(new SensorReport("s" + i, "l" + i, "z" + i,
					false, true, i + 1));
		}

		valveReports = new LinkedList<ValveReport>();
		for (int i = 0; i < 3; i++) {
			valveReports.add(new ValveReport("v" + i, "l" + i, "z" + i, false,
					false, true, i + 1, 0));
		}
	}

	@Test
	public void testFailureReport() {
		// Tests a failure report based on existing collections of
		// sensor and valve reports.

		FailureReport report = new FailureReport(sensorReports, valveReports);

		assertEquals(sensorReports, report.failedSensors);
		assertEquals(valveReports, report.failedValves);

		for (SensorReport r : sensorReports) {
			assertTrue(report.failedSensors.contains(r));
		}

		for (ValveReport r : valveReports) {
			assertTrue(report.failedValves.contains(r));
		}
	}

	@Test
	public void testFailureReportEmtpyCollections() {
		// Tests a failure report based on empty collections

		FailureReport report = new FailureReport(
				sensorReports = new LinkedList<SensorReport>(),
				valveReports = new LinkedList<ValveReport>());

		assertEquals(sensorReports, report.failedSensors);
		assertEquals(valveReports, report.failedValves);

		assertTrue(sensorReports.isEmpty());
		assertTrue(valveReports.isEmpty());
	}

	@Test
	public void testFailureReportNullArguments() {
		// FailureReport's constructor receiving null as arguments.

		FailureReport report = new FailureReport(null, null);

		assertNull(report.failedSensors);
		assertNull(report.failedValves);
	}

}
