package simulation;

import static tools.HelperFunctions.*;
import static org.junit.Assert.*;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

import org.junit.Test;

public class SimControlPanelTest {

	@Test
	public void testSimControlPanel() throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Tests SimControlPanel's initial state.
		
		SimControlPanel panel = new SimControlPanel();

		SimDisplay display = extractField(panel, "display");
		SimKeypad keypad = extractField(panel, "keypad");

		assertNotNull(display);
		assertNotNull(keypad);
		assertEquals(display, panel.getDisplay());

		int foundKeypad = 0;
		int foundDisplay = 0;

		// Test whether the display and the keypad are
		// added to the ControlPanel's components
		for (Component c : panel.getComponents()) {
			if (c.equals(keypad))
				foundKeypad++;
			if (c.equals(display))
				foundDisplay++;
		}

		assertEquals(1, foundKeypad);
		assertEquals(1, foundDisplay);
		
		// No ActionListeners should be registered.
		JButton[] keypadButtons = extractField(keypad, "key");
		for (JButton b : keypadButtons) {
			assertEquals(0, b.getActionListeners().length);
		}

		JButton[] displayButtons = extractField(display, "scrnBtn");
		for (JButton b : displayButtons) {
			assertEquals(0, b.getActionListeners().length);
		}

	}

	@Test
	public void testSetKeypadListener() throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Tests setKeypadListener().

		SimControlPanel panel = new SimControlPanel();

		ActionListener mockListener = new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
			}
		};

		panel.setKeypadListener(mockListener);

		SimKeypad pad = extractField(panel, "keypad");
		JButton[] buttons = extractField(pad, "key");

		// All keypad buttons should have the supplied listener
		// registered as their listener.
		for (JButton b : buttons) {
			int foundListeners = 0;
			for (ActionListener l : b.getActionListeners())
				if (l.equals(mockListener))
					foundListeners++;
			assertEquals(1, foundListeners);
		}

	}

	@Test
	public void testSetScreenButtonListener() throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Tests setScreenButtonListener().
		
		SimControlPanel panel = new SimControlPanel();

		ActionListener mockListener = new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
			}
		};

		panel.setScreenButtonListener(mockListener);

		SimDisplay pad = extractField(panel, "display");
		JButton[] buttons = extractField(pad, "scrnBtn");

		// All screen buttons should have the supplied listener
		// registered as their listener.
		for (JButton b : buttons) {
			int foundListeners = 0;
			for (ActionListener l : b.getActionListeners())
				if (l.equals(mockListener))
					foundListeners++;
			assertEquals(1, foundListeners);
		}
	}

}
