package simulation;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static tools.HelperFunctions.extractField;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;
import java.util.HashSet;

import javax.swing.JButton;
import javax.swing.JPanel;

import org.junit.Test;

public class SimKeypadTest {

	@Test
	public void testSimKeypad() throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Tests the initial state of a SimKeypad.
		SimKeypad pad = new SimKeypad();
		
		HashSet<String> keyLabels = new HashSet<String>();
		for (int i=0; i < 10; i++){
			keyLabels.add(new Integer(i).toString());
		}
		keyLabels.add("DEL");
		keyLabels.add("ESC");

		// Tests internal constants

		int rows = extractField(pad, "NUM_ROWS");
		int columns = extractField(pad, "NUM_COLUMNS");
		assertEquals(12, rows * columns);

		// Does the internal panel keyPanel exist?

		JPanel panel = extractField(pad, "keyPanel");
		assertNotNull(panel);

		// Is keyPanel one of pad's components?
		int contained = 0;
		for (Component p : pad.getComponents())
			if (p.equals(panel))
				contained++;

		assertEquals(1, contained);

		// Are rows * columns buttons created,
		// which all have no attached ActionListeners,
		// but are contained in keyPanel
		JButton[] keys = extractField(pad, "key");
		assertEquals(rows * columns, keys.length);
		for (JButton b : keys) {
			assertNotNull(b);
			assertTrue(keyLabels.contains(b.getText()));
			keyLabels.remove(b.getText());
			assertEquals(0, b.getActionListeners().length);
			Component[] comp = panel.getComponents();
			contained = 0;
			for (Component c : comp)
				if (c.equals(b))
					contained++;
			assertEquals(1, contained);
		}

	}

	@Test
	public void testSetListener() throws IllegalArgumentException,
			SecurityException, IllegalAccessException, NoSuchFieldException {
		// Tests setListener().
		
		SimKeypad pad = new SimKeypad();

		ActionListener l = new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
			}
		};

		pad.setListener(l);

		// An action listener set via setListener()
		// should be attached to all buttons.
		JButton[] keys = extractField(pad, "key");
		for (JButton b : keys) {
			EventListener[] e = b.getActionListeners();
			int contained = 0;
			for (EventListener el : e) {
				if (el.equals(l))
					contained++;
			}
			assertEquals(1, contained);
		}

	}

}
