package simulation;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import tools.TestObserver;

public class SimSensorTest {

	private TestObserver observer;

	@Before
	public void init() {
		observer = new TestObserver();
	}

	@Test
	public void testSimSensorConstructor() {
		// SimSensor's constructor.

		SimSensor sensor = new SimSensor(null);

		assertEquals(null, sensor.getName());
		assertEquals(40, sensor.getLevel());
		assertFalse(sensor.getIsFailed());

		sensor = new SimSensor("sensor1");

		assertEquals("sensor1", sensor.getName());
		assertEquals(40, sensor.getLevel());
		assertFalse(sensor.getIsFailed());
	}

	@Test
	public void testGetLevel() {
		// Tests getLevel(). The Sensor has not failed.

		SimSensor sensor = new SimSensor("name");
		sensor.setLevel(0);
		assertEquals(0, sensor.getLevel());

		sensor.setLevel(50);
		assertEquals(50, sensor.getLevel());
	}
	
	@Test
	public void testGetLevelSensorFailed() {
		// Tests getLevel(). Sensor has failed
		
		SimSensor sensor = new SimSensor("name");

		sensor.setIsFailed();
		assertEquals(-1, sensor.getLevel());

		sensor.setLevel(75);
		assertEquals(-1, sensor.getLevel());
	}

	@Test
	public void testGetLevelSensorReset() {
		// Tests getLevel(). Sensor is reset, so
		// no longer failing
		
		SimSensor sensor = new SimSensor("name");
		sensor.setLevel(75);

		sensor.setIsFailed();
		sensor.resetIsFailed();
		assertEquals(75, sensor.getLevel());

		sensor.setLevel(100);
		assertEquals(100, sensor.getLevel());

	}

	@Test
	public void testSetLevel() {
		// Tests setLevel().

		SimSensor sensor = new SimSensor("sensor");
		sensor.addObserver(observer);

		sensor.setLevel(0);
		assertEquals(0, sensor.getLevel());
		assertEquals(1, observer.notifications());

		sensor.setLevel(100);
		assertEquals(100, sensor.getLevel());
		assertEquals(2, observer.notifications());

		sensor.setLevel(100);
		assertEquals(100, sensor.getLevel());
		// Since the sensor's state has not changed,
		// the observer should not be notified.
		assertEquals(2, observer.notifications());
	}

	@Test
	public void testSetLevelIllegalArguments() {
	
		// Tests setLevel(). Its precondition is violated.
		
		SimSensor sensor = new SimSensor("sensor");
		sensor.addObserver(observer);

		try {
			sensor.setLevel(-1);
			fail();
		} catch (IllegalArgumentException e) {
			
		}

		try {
			sensor.setLevel(Integer.MIN_VALUE);
			fail();
		} catch (IllegalArgumentException e) {
			
		}

		try {
			sensor.setLevel(Integer.MAX_VALUE);
			fail();
		} catch (IllegalArgumentException e) {
			
		}

		try {
			sensor.setLevel(101);
			fail();
		} catch (IllegalArgumentException e) {
			
		}

		assertEquals(40, sensor.getLevel());
		assertEquals(0, observer.notifications());
	}

	@Test
	public void testSetIsFailed() {
		// Tests setIsFailed().
		
		SimSensor sensor = new SimSensor("name");
		sensor.addObserver(observer);

		sensor.setIsFailed();
		assertTrue(sensor.getIsFailed());
		assertEquals(1, observer.notifications());

		sensor.setIsFailed();
		assertTrue(sensor.getIsFailed());
		// Since the sensor has already failed, its
		// state did not change, so no additional notification
		// is expected.
		assertEquals(1, observer.notifications());
	}

	@Test
	public void testResetIsFailed() {
		// Tests resetIsFailed().
		
		SimSensor sensor = new SimSensor("name");
		sensor.addObserver(observer);

		sensor.resetIsFailed();
		assertFalse(sensor.getIsFailed());
		// The sensor's state did not change,
		// so notification is expected
		assertEquals(0, observer.notifications());

		sensor = new SimSensor("name2");
		observer = new TestObserver();
		sensor.addObserver(observer);

		sensor.setIsFailed();
		sensor.resetIsFailed();

		assertFalse(sensor.getIsFailed());
		assertEquals(2, observer.notifications());

	}

}
