package tools;

import static org.junit.Assert.assertEquals;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Observable;
import java.util.Observer;

public class HelperFunctions {

	// Extract object's fields
	@SuppressWarnings("unchecked")
	public static <T> T extractField(Object obj, String argName)
			throws IllegalArgumentException, IllegalAccessException,
			SecurityException, NoSuchFieldException {
		Field intervalField = obj.getClass().getDeclaredField(argName);
		intervalField.setAccessible(true);
		return (T) intervalField.get(obj);
	}

	@SuppressWarnings("unchecked")
	public static <T> T extractStaticField(@SuppressWarnings("rawtypes") Class c, String argName)
			throws SecurityException, NoSuchFieldException,
			IllegalArgumentException, IllegalAccessException {
		Field intervalField = c.getDeclaredField(argName);
		intervalField.setAccessible(true);
		return (T) intervalField.get(null);
	}

	// Extract fields as Fields

	public static Field extractFieldAsField(Object obj, String argName)
			throws IllegalArgumentException, IllegalAccessException,
			SecurityException, NoSuchFieldException {
		return obj.getClass().getDeclaredField(argName);
	}

	public static Field extractStaticFieldAsField(@SuppressWarnings("rawtypes") Class c, String argName)
			throws IllegalArgumentException, IllegalAccessException,
			SecurityException, NoSuchFieldException {
		return c.getDeclaredField(argName);
	}

	// Checks whether method "method" of object obj is synchronized
	@SuppressWarnings("unchecked")
	public static boolean isSynchronized(@SuppressWarnings("rawtypes") Class c, String method,
			@SuppressWarnings("rawtypes") Class[] arguments) throws SecurityException, NoSuchMethodException {
		return ((c.getMethod(method, arguments).getModifiers() & Modifier.SYNCHRONIZED) != 0);
	}

	// Tests whether an Observable is observed by a specific observer
	public static void assertObserved(Observable observable, Observer observer) {
		assertEquals(1, observable.countObservers());
		observable.deleteObserver(observer);
		assertEquals(0, observable.countObservers());
	}

}
